/**
 *  Copyright (c) 2014, 
 *  BTC Embedded Systems AG, Oldenburg, Germany  
 *  All rights reserved
 *  --------------------------------------------------------- 
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/
package com.btc.ep.dal.persist.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;


import com.btc.ep.base.bl.impl.*;
import com.btc.ep.dal.persist.PersistenceManager;
import com.btc.ep.dal.provider.PersistenceManagerProvider;
import com.btc.ep.dal.provider.impl.OdbPersistenceManagerProvider;

/**
 * Database handling.
 * 
 * @author jacek
 * @author Simon Kollmann
 * 
 */
public class PersistenceManagerImpl extends PersistenceManager {

//    private static Logger log = LoggerFactory.getLogger(PersistenceManager.class);

    /**
     * System property for setting database provider
     */
    public static final String DATABASE_ARG = "epp.database";

    /**
     * Hack for default database provider
     */
    private static final String DEFAULT_DB = OdbPersistenceManagerProvider.ID;

    /**
     * PersistenceService singleton
     */
    private PersistenceService service;

    /**
     * registered DB providers
     */
    private Map<String, PersistenceManagerProvider> providers = new HashMap<>();

    /**
     * DB provider currently in use
     */
    private PersistenceManagerProvider provider;

    /**
     * Constructor
     *
     */
    public PersistenceManagerImpl() {

    }

    /**
     * activate component.
     * <p>DS - wiring up.
     *
     */
    protected void activate() {
        //Noting to do to activate this component
    }

    /**
     * add a provider
     * <p>DS - wiring up.
     *
     * @param provider db specific provider
     */
    public void addProvider(PersistenceManagerProvider provider) {
        providers.put(provider.getId(), provider);
    }

    /**
     * remove a provider
     * <p>DS - wiring up.
     *
     * @param provider db specific provider
     */
    public void removeProvider(PersistenceManagerProvider provider) {
        providers.remove(provider.getId());
    }

    /**
     * Get Persistence Service.
     * <p>initializes and opens DB connection on first call
     * 
     * @return the PersistenceService (a generic DAO)
     */
    @Override
    public PersistenceService getPersistenceService() {
        getProvider();

        if (service == null) {
            EntityManagerFactory emFactory = provider.getEntityManagerFactory();
Thread.currentThread().setContextClassLoader(ProfileImpl.class.getClassLoader());           
            this.service = new PersistenceService(emFactory.createEntityManager());
        }
        //log.debug("Database path: {}", provider.getDBPath().getAbsolutePath());

        return service;
    }

    /**
     * Return the actual Provider or select the by System-Property <code>DATABASE_ARG</code> configured Provider
     *
     * @return Provider for this instance
     */
    protected PersistenceManagerProvider getProvider() {
        if (this.provider == null) {
            String db = System.getProperty(DATABASE_ARG);
            this.provider = providers.get(db);
            //fallback to default
            if (this.provider == null) {
                this.provider = providers.get(DEFAULT_DB);
            }

            if (this.provider == null) {
                throw new IllegalStateException("PersistenceManagerProvider is not properly setup. "
                    + "Maybe you misspelled the >-Depp.database< property?");
            }
        }
        return this.provider;
    }

    @Override
    public synchronized void backup(File backupFile) throws Exception {
        getProvider().backup(backupFile);

    }

    @Override
    public void close() {
        if (this.service != null) {
            this.service.close();
            this.service = null;
        }
        getProvider().close();
    }

    @Override
    public void clear() {
        close();
        getProvider().clearFiles();
    }

    @Override
    public void open(File path) throws Exception {
        close();
        getProvider().openFile(path);
    }

}
