/**
 *  Copyright (c) 2014, 
 *  BTC Embedded Systems AG, Oldenburg, Germany  
 *  All rights reserved
 *  --------------------------------------------------------- 
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/

package com.btc.ep.dal.provider.impl;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.StandardCopyOption;
import java.nio.file.attribute.BasicFileAttributes;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import com.btc.ep.dal.provider.PersistenceManagerProvider;

/**
 * Provider for objectDB.
 * 
 * 
 */
public class OdbPersistenceManagerProvider implements PersistenceManagerProvider {

    //private static Logger log = LoggerFactory.getLogger(OdbPersistenceManagerProvider.class);

    public static final String ID = "objectDB";
    
    //private static final String PERSISTENCE_UNIT_ODB = "eppdb.odb";
    private static final String PERSISTENCE_UNIT_ODB = "eppdb.odb";
    
    private static final String DB_FILE = "eppdb.odb";

    private EntityManagerFactory factory;
    private File dbPath;

    @Override
    public String getId() {
        return ID;
    }

    @Override
    public EntityManagerFactory getEntityManagerFactory() {
        if (factory == null) {
            this.factory = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_ODB);
        }
        return this.factory;
    }

    @Override
    public File getDBPath() {
        if (factory != null) {
            String dbpath = (String)factory.getProperties().get("objectdb.connection.path");
            this.dbPath = new File(dbpath);
        } else {
            String dbpath = System.getProperty("objectdb.home");
            if (dbpath != null) {
                this.dbPath = new File(dbpath, DB_FILE);
            }
        }
        return this.dbPath;
    }

    @Override
    public void backup(final File backupFile) throws IOException {

        final File dbFile = getDBPath();

        if (this.factory != null) {
            EntityManager em = factory.createEntityManager();
            TypedQuery<Thread> backupQuery = em.createQuery("objectdb backup", Thread.class);

            Path backupDir = Files.createTempDirectory(dbFile.getParentFile().toPath(), "BAK");
            backupQuery.setParameter("target", backupDir.toFile());
            Thread bt = (Thread)backupQuery.getSingleResult();
            try {
                bt.join();
            } catch (InterruptedException e) {
                // TODO Implement exception handling

            }
            em.close();
            Files.walkFileTree(backupDir, new SimpleFileVisitor<Path>() {

                @Override
                public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                    if (file.getFileName().toString().equals(dbFile.getName())) {
                        Files.copy(file, backupFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
                    }
                    Files.delete(file);
                    return FileVisitResult.CONTINUE;
                }

                @Override
                public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
                    Files.delete(dir);
                    return FileVisitResult.CONTINUE;
                }
            });

        } else {
            Files.copy(dbFile.toPath(), backupFile.toPath(), StandardCopyOption.REPLACE_EXISTING);
        }
    }

    @Override
    public void close() {
        if (factory == null) {
            return;
        }
        this.factory.close();
        this.factory = null;
    }

    @Override
    public void clearFiles() {
        File path = getDBPath();
        close();
        if (path != null) {
            File path2 = new File(path.getPath() + "$");
            try {
                Files.deleteIfExists(path.toPath());
                Files.deleteIfExists(path2.toPath());
            } catch (IOException e) {
                //log.warn("can't delete {}", path);
            }
        }
    }

    @Override
    public void openFile(File path) throws Exception {
        File dbpath = getDBPath();
        Files.copy(path.toPath(), dbpath.toPath());
    }

}
