/**
 *  Copyright (c) 2014, 
 *  BTC Embedded Systems AG, Oldenburg, Germany  
 *  All rights reserved
 *  --------------------------------------------------------- 
 *  $Author: $
 *  $Date:  $
 *  $Revision: $
 **/

package com.btc.ep.dal.tests.ut;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.File;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.mockito.Mockito;

import com.btc.ep.dal.provider.PersistenceManagerProvider;
import com.btc.ep.dal.persist.PersistenceManager;
import com.btc.ep.dal.persist.impl.PersistenceManagerImpl;
import com.btc.ep.dal.persist.impl.PersistenceService;

/**
 * This class tests base functions of persistence manager.
 * 
 * @author me
 */
public class UT_PersistenceManagerImpl {

    PersistenceManager persistenceManager;
    PersistenceManagerProvider pmp;

    /**
     * Brief Function description
     *
     * @throws java.lang.Exception anything
     */
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        // nothing to do BeforeClass at for 
    }

    /**
     * Brief Function description
     *
     * @throws java.lang.Exception anything
     */
    @AfterClass
    public static void tearDownAfterClass() throws Exception {
    }

    /**
     * Brief Function description
     *
     * @throws java.lang.Exception anything
     */
    @Before
    public void setUp() throws Exception {

        pmp = Mockito.mock(PersistenceManagerProvider.class);
        Mockito.when(pmp.getId()).thenReturn("EPPDataBase");
        System.setProperty("epp.database", "EPPDataBase");
        
        System.out.println(System.getProperty("user.dir"));
        
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("eppdb");
        Mockito.when(pmp.getEntityManagerFactory()).thenReturn(entityManagerFactory);

        String dbpath = System.getProperty("objectdb.home");
        File dbPath = new File(dbpath, "eppdb.odb");
        Mockito.when(pmp.getDBPath()).thenReturn(dbPath);

        // create PersistenceManager 
        PersistenceManagerImpl persistenceManagerImpl = new PersistenceManagerImpl();
        persistenceManagerImpl.addProvider(pmp);
        persistenceManager = persistenceManagerImpl;

    }

    /**
     * Brief Function description
     *
     * @throws java.lang.Exception anything
     */
    @After
    public void tearDown() throws Exception {
    }

    /**
     * Test method for {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#getPersistenceService()}.
     */
    @Test
    public void testGetPersistenceService() {

        PersistenceService ps1 = persistenceManager.getPersistenceService();
        assertNotNull("could create PersistentsService", ps1);

        PersistenceService ps2 = persistenceManager.getPersistenceService();
        assertEquals("get PersistentsService second time", ps1, ps2);

    }

    /**
     * Test method for {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#backup(java.io.File)}.
     */
    @Test
    public void testBackup() {

        File backupFile = new File("test.epp");

        //test
        try {
            persistenceManager.backup(backupFile);
        } catch (Exception e) {
            fail("unexpected Exception");
        }

        // now check if method backup was called with the parameter backupFile
        try {
            Mockito.verify(pmp).backup(backupFile);
        } catch (Exception e) {
            fail("unexpected Exception");
        }

        //test with null
        try {
            persistenceManager.backup(null);
        } catch (Exception e) {
            fail("unexpected Exception");
        }
    }

    /**
     * Test method for {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#clear()}.
     */
    @Test
    public void testClear() {

        //test
        persistenceManager.clear();

        // now check if clear() call all required methods of the PersistenceManagerProvider & PersistenceService
        Mockito.verify(pmp).close();
        Mockito.verify(pmp).clearFiles();

        //TODO verify modification on the PersistenceService not possible
    }

    /**
     * Test method for {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#close()}.
     */
    @Test
    public void testClose() {

        //test
        persistenceManager.close();

        // now check if close() call all required methods of the PersistenceManagerProvider & PersistenceService
        Mockito.verify(pmp).close();

        //TODO verify modification on the PersistenceService not possible
    }

    /**
     * Test method for {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#open(java.io.File)}.
     */
    @Test
    public void testOpen() {

        File eppFile = new File("test.epp");

        //test
        try {
            persistenceManager.open(eppFile);
        } catch (Exception e) {
            fail("unexpected Exception");
        }

        // now check if close() call all required methods of the PersistenceManagerProvider & PersistenceService
        Mockito.verify(pmp).close();
        try {
            Mockito.verify(pmp).openFile(eppFile);
        } catch (Exception e) {
            fail("unexpected Exception");
        }

        //TODO verify modification on the PersistenceService not possible
    }

    /**
     * Test method for
     * {@link com.btc.ep.dal.persist.impl.PersistenceManagerImpl#addProvider(com.btc.ep.dal.PersistenceManagerProvider)}
     * .
     */
    @Test
    public void testAddProvider() {

        // create PersistenceManager 
        PersistenceManagerImpl persistenceManagerImpl = new PersistenceManagerImpl();
        persistenceManagerImpl.addProvider(pmp);
        PersistenceManager persistenceManager = persistenceManagerImpl;

        PersistenceService ps1 = persistenceManager.getPersistenceService();

        assertNotNull("could add PersistentsService", ps1);
    }
}
